/**
 * @brief  PD_LgtSpdLmt: sign speed limit and curve speed limit
 * @param
 * @author lzc
 * @date 2022-4-20
 * @version 1.0
 * @property
 * */

#pragma once
// #ifndef __PD_LGTSPDLMT_H_
// #define __PD_LGTSPDLMT_H_

#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include "PD_Input.h"
#include "../../source/PD_commonfunc.h"
#include "PD_LgtTrajectory.h"
#include "glog/logging.h"
// #include "mpc_drv_msgs/msg/behavior_node_local.hpp"
#include "../../../include/CtApBehaviorAnalysis_/interface/pd_interface.h"
// #include "prediction.h"

using namespace ADC::PD_LgtTrj;
// using namespace ADC;

// #endif

namespace ADC
{
    namespace PD_LgtSpdLmt
    {

        struct Sample_ST
        {
            float t;
            float a;
            float s_end;
        };
        struct test_STInfo
        {
            // LgtTraj_Coefficient LgtTrjj_ST_coe;
            // std::vector<PD_LgtTrj_ST_point> pd_LgtTrj_ST_trace;
            std::vector<LgtTraj_Coefficient> lgttrj_st_coe;
            std::vector<PD_LgtTrj_ST_point> pd_lgttrj_st_trace;
        };

        struct LastCyc_SpdLmtInfo
        {
            uint8_t LastCyc_SpdLmtId = 0;
            uint8_t LastCyc_SpdLmtType= 0;
            bool LastCyc_SpdLmtFlg = 0;
            // uint8_t LastCyc_SpdLmtId = 0;
            
        };
       

        // struct LgtSpdLmt_Coefficient
        // {
        //     float tm;
        //     float c0;
        //     float c1;
        //     float c2;
        //     float c3;
        //     float c4;
        //     float c5;
        // };

        // struct LgtSpdLmt_TrjPnt
        // {
        //     float t;   // unit:s
        //     float s;   // unit：m
        //     float vx;  // unit:m/s
        //     float ax;  // unit:m/s2
        //     float jkx; // unit:m/s3
        // };

        // struct LgtSpdLmt_SampJk
        // {
        //     float t = 0.0f;
        //     // float lgtaccval = 0.0f;
        //     float jerkval = 0.0f;
        // };

        // struct LgtSpdLmt_JkInfo
        // {
        //     std::vector<LgtSpdLmt_Coefficient> pd_lgtspdlmt_coe;
        //     // std::vector<LgtSpdLmt_SampJk> pd_lgtspdlmt_jerk;
        //     std::vector<Sample_ST_Jerkpnt> pd_lgtspdlmt_jerk
        // };

        // struct LgtSpdLmt_TrjInfo
        // {
        //     LgtSpdLmt_Coefficient pd_lgtspdlmt_coe;
        //     std::vector<LgtSpdLmt_TrjPnt> pd_lgtspdlmt_st_TrjPnt;
        // };

        class LgtSpdLmt
        {
        public:
            LgtSpdLmt(const PD::PD_Input &pd_input,const ADC::PD_LgtSpdLmt::LastCyc_SpdLmtInfo &spdlmtinfo);
            ~LgtSpdLmt() = default;

            // curve pre-deceleration calculation
            void CurvSpdLmtCalu();

            void JudgeSpdLmt_ThrsDistCalu(const PD::PD_Input &pd_input,
            const ADC::PD_LgtSpdLmt::LastCyc_SpdLmtInfo &spdlmtinfo);
            // sign01 info
            uint8_t SlecSpdLmt_id = 0;
            float SlecSpdVal = 0.0f;
            float SlecSpdDist = 0.0f;
            float SignSpdLimt_Updist = 0.0f;
            float SignSpdLimt_Lowdist = 0.0f;
            // float

            // sign speed limit calculation
            void SignSpdLmtCalu_Coe01(const PD::PD_Input &pd_input);

            void SignSpdLmtCalu_Coe02(const PD::PD_Input &pd_input);

            void Sign_OptmalSpdLmtSlect(const PD::PD_Input &pd_input);

            void SignSpdLmt_OptmalTrj(const PD::PD_Input &pd_input);

            void DistSampCalu_01(const PD::PD_Input &pd_input);
            void DistSampCalu_02(const PD::PD_Input &pd_input);
            std::vector<float> LgtSpdLmt_JerkCost(const std::vector<LgtTraj_Coefficient> &lgtspdlmt_coevec);

            // LgtSpdLmt trajectory point calculation
            void LgtSpdLmt_ST_trace();
            void process(const PD::PD_Input &pd_input);

            // LgtSpdLmt output
            float LgtSpdLmt_DistVal01 = 0.0f;
            float LgtSpdLmt_DistVal02 = 0.0f;

            float LgtSpdLmt_DistVal = 0.0f;
            float LgtSpdLmt_SpdVal = 0.0f;
            bool LgtSpdLmt_SpdLmtFlg = false;
            uint8_t LgtSpdLmt_type = 0;

            int OptimalSignSpdLmt_Indx = 0;

            // LgtSpdLmt_TrjInfo pd_lgtspdlmt_st_trace;
            PD_LgtTrj_STInfo pd_lgtspdlmt_st_trace;

            std::vector<float> LgtSpdLmtAvgAccProc();

            LastCyc_SpdLmtInfo lastcyc_spdlmtinfo;
            uint8_t SpdLmtType2MatchID(const ADC::PD_LgtSpdLmt::LastCyc_SpdLmtInfo &spdlmtinfo);

            // void LgtSpdLmtInfoMatchProc(const ADC::PD_LgtSpdLmt::LastCyc_SpdLmtInfo &spdlmtinfo); // match lgtspdval and lgtspddist with lgtspdflg and type

        private:
            // sign02 info
            uint8_t SlecSpdLmt_id_2nd = 0;
            float SlecSpdVal_2nd = 0.0f;
            float SlecSpdDist_2nd = 0.0f;
            float SignSpdLimt_Updist_2nd = 0.0f;
            float SignSpdLimt_Lowdist_2nd = 0.0f;

            std::vector<float> s_end_vec01;
            std::vector<float> s_end_vec02;

            std::vector<Sample_ST> Samp_Dist_Pnt01;
            std::vector<Sample_ST> Samp_Dist_Pnt02;

            std::vector<LgtTraj_Coefficient> lgtspdlmt_coevec01;
            std::vector<LgtTraj_Coefficient> lgtspdlmt_coevec02;

            // std::vector<LgtSpdLmt_SampJk> lgtspdlmt_jkinfovec01;
            std::vector<Sample_ST_Jerkpnt> lgtspdlmt_jkinfovec01;

            std::vector<float> pd_lgtspdlmt_jerkcost01;
            std::vector<float> pd_lgtspdlmt_jerkcost02;
            bool SignSpdLmtCalu_Trg01 = false;
            bool SignSpdLmtCalu_Trg02 = false;

            bool HdSignSpdLmtCalu_Trg01 = false;

            bool CurvSpdLmtCalu_Trg = false;

            int LgtSpdLmt01_MinJerkValIndx = 0;
            bool LgtSpdLmt01_StsFlag = false;

            int LgtSpdLmt02_MinJerkValIndx = 0;
            bool LgtSpdLmt02_StsFlag = false;
        };
    } // namespace PD_LgtSpdLmt
} // namespace ADC